@@ -69,7 +69,6 @@ void TestPointerConstraints::init()
     m_display->createShm();
     m_seatInterface = new SeatInterface(m_display, m_display);
     m_seatInterface->setHasPointer(true);
-    m_seatInterface->create();
     m_compositorInterface = new CompositorInterface(m_display, m_display);
     m_pointerConstraintsInterface = new PointerConstraintsV1Interface(m_display, m_display);
 
@@ -224,12 +223,14 @@ void TestPointerConstraints::testLockPointer()
     m_seatInterface->setFocusedPointerSurface(serverSurface);
     QSignalSpy pointerMotionSpy(m_pointer, &Pointer::motion);
     QVERIFY(pointerMotionSpy.isValid());
-    m_seatInterface->setPointerPos(QPoint(0, 1));
+    m_seatInterface->notifyPointerMotion(QPoint(0, 1));
+    m_seatInterface->notifyPointerFrame();
     QVERIFY(pointerMotionSpy.wait());
 
     serverLockedPointer->setLocked(true);
     QCOMPARE(serverLockedPointer->isLocked(), true);
-    m_seatInterface->setPointerPos(QPoint(1, 1));
+    m_seatInterface->notifyPointerMotion(QPoint(1, 1));
+    m_seatInterface->notifyPointerFrame();
     QCOMPARE(lockedChangedSpy.count(), 1);
     QCOMPARE(pointerMotionSpy.count(), 1);
     QVERIFY(lockedSpy.isEmpty());
@@ -256,7 +257,8 @@ void TestPointerConstraints::testLockPointer()
     QCOMPARE(lockedSpy.count(), 1);
 
     // now motion should work again
-    m_seatInterface->setPointerPos(QPoint(0, 1));
+    m_seatInterface->notifyPointerMotion(QPoint(0, 1));
+    m_seatInterface->notifyPointerFrame();
     QVERIFY(pointerMotionSpy.wait());
     QCOMPARE(pointerMotionSpy.count(), 2);
 
@@ -360,7 +362,7 @@ void TestPointerConstraints::testConfinePointer()
 
 enum class Constraint {
     Lock,
-    Confine
+    Confine,
 };
 
 Q_DECLARE_METATYPE(Constraint)
