@@ -3,8 +3,7 @@
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
-#ifndef KWAYLAND_SERVER_IDLE_INTERFACE_P_H
-#define KWAYLAND_SERVER_IDLE_INTERFACE_P_H
+#pragma once
 
 #include "idle_interface.h"
 
@@ -55,5 +54,3 @@ protected:
 };
 
 }
-
-#endif
