@@ -23,16 +23,18 @@
 namespace KWaylandServer
 {
 
-static const quint32 s_version = 13;
+static const quint32 s_version = 14;
 
 class PlasmaWindowManagementInterfacePrivate : public QtWaylandServer::org_kde_plasma_window_management
 {
 public:
     PlasmaWindowManagementInterfacePrivate(PlasmaWindowManagementInterface *_q, Display *display);
     void sendShowingDesktopState();
-    void sendStackingOrderChanged();
     void sendShowingDesktopState(wl_resource *resource);
+    void sendStackingOrderChanged();
     void sendStackingOrderChanged(wl_resource *resource);
+    void sendStackingOrderUuidsChanged();
+    void sendStackingOrderUuidsChanged(wl_resource *resource);
 
     PlasmaWindowManagementInterface::ShowingDesktopState state = PlasmaWindowManagementInterface::ShowingDesktopState::Disabled;
     QList<PlasmaWindowInterface*> windows;
@@ -73,10 +75,10 @@ public:
     PlasmaWindowManagementInterface *wm;
 
     bool unmapped = false;
-    bool destroyed = false;
     PlasmaWindowInterface *parentWindow = nullptr;
     QMetaObject::Connection parentWindowDestroyConnection;
     QStringList plasmaVirtualDesktops;
+    QStringList plasmaActivities;
     QRect geometry;
     PlasmaWindowInterface *q;
     QString m_title;
@@ -92,7 +94,6 @@ public:
 
 protected:
     void org_kde_plasma_window_bind_resource(Resource *resource) override;
-    void org_kde_plasma_window_destroy_resource(Resource *resource) override;
     void org_kde_plasma_window_set_state(Resource *resource, uint32_t flags, uint32_t state) override;
     void org_kde_plasma_window_set_virtual_desktop(Resource *resource, uint32_t number) override;
     void org_kde_plasma_window_set_minimized_geometry(Resource *resource, wl_resource *panel, uint32_t x, uint32_t y, uint32_t width, uint32_t height) override;
@@ -105,6 +106,8 @@ protected:
     void org_kde_plasma_window_request_enter_virtual_desktop(Resource *resource, const QString &id) override;
     void org_kde_plasma_window_request_enter_new_virtual_desktop(Resource *resource) override;
     void org_kde_plasma_window_request_leave_virtual_desktop(Resource *resource, const QString &id) override;
+    void org_kde_plasma_window_request_enter_activity(Resource *resource, const QString &id) override;
+    void org_kde_plasma_window_request_leave_activity(Resource *resource, const QString &id) override;
 };
 
 PlasmaWindowManagementInterfacePrivate::PlasmaWindowManagementInterfacePrivate(PlasmaWindowManagementInterface *_q, Display *display)
@@ -153,13 +156,31 @@ void PlasmaWindowManagementInterfacePrivate::sendStackingOrderChanged(wl_resourc
     }
 
     send_stacking_order_changed(r, QByteArray::fromRawData(reinterpret_cast<const char*>(stackingOrder.constData()), sizeof(uint32_t) * stackingOrder.size()));
+}
+
+void PlasmaWindowManagementInterfacePrivate::sendStackingOrderUuidsChanged()
+{
+    const auto clientResources = resourceMap();
+    for (auto resource : clientResources) {
+        sendStackingOrderUuidsChanged(resource->handle);
+    }
+}
+
+void PlasmaWindowManagementInterfacePrivate::sendStackingOrderUuidsChanged(wl_resource *r)
+{
+    if (wl_resource_get_version(r) < ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STACKING_ORDER_UUID_CHANGED_SINCE_VERSION) {
+        return;
+    }
 
     QString uuids;
     for (const auto &uuid : qAsConst(stackingOrderUuids)) {
         uuids += uuid;
-        uuids += QStringLiteral(";");
+        uuids += QLatin1Char(';');
+    }
+    // Remove the trailing ';', on the receiving side this is interpreted as an empty uuid.
+    if (stackingOrderUuids.size() > 0) {
+        uuids.remove(uuids.length() - 1, 1);
     }
-
     send_stacking_order_uuid_changed(r, uuids);
 }
 
@@ -173,6 +194,7 @@ void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_bi
         }
     }
     sendStackingOrderChanged(resource->handle);
+    sendStackingOrderUuidsChanged(resource->handle);
 }
 
 void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_show_desktop(Resource *resource, uint32_t state)
@@ -188,7 +210,7 @@ void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_sh
         s = PlasmaWindowManagementInterface::ShowingDesktopState::Disabled;
         break;
     }
-    emit q->requestChangeShowingDesktop(s);
+    Q_EMIT q->requestChangeShowingDesktop(s);
 }
 
 void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_get_window(Resource *resource, uint32_t id, uint32_t internal_window_id)
@@ -199,10 +221,9 @@ void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_ge
             return;
         }
     }
-    // create a temp window just for the resource and directly send an unmapped
-    PlasmaWindowInterface *window = new PlasmaWindowInterface(q, q);
-    window->d->unmapped = true;
-    window->d->add(resource->client(), id, resource->version());
+    // create a temp window just for the resource, bind then immediately delete it, sending an unmap event
+    PlasmaWindowInterface window(q, q);
+    window.d->add(resource->client(), id, resource->version());
 }
 
 void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_get_window_by_uuid(Resource *resource, uint32_t id, const QString &internal_window_uuid)
@@ -214,10 +235,9 @@ void PlasmaWindowManagementInterfacePrivate::org_kde_plasma_window_management_ge
     );
     if (it == windows.constEnd()) {
         qCWarning(KWAYLAND_SERVER) << "Could not find window with uuid" << internal_window_uuid;
-        // create a temp window just for the resource and directly send an unmapped
-        PlasmaWindowInterface *window = new PlasmaWindowInterface(q, q);
-        window->d->unmapped = true;
-        window->d->add(resource->client(), id, resource->version());
+        // create a temp window just for the resource, bind then immediately delete it, sending an unmap event
+        PlasmaWindowInterface window(q, q);
+        window.d->add(resource->client(), id, resource->version());
         return;
     }
     (*it)->d->add(resource->client(), id, resource->version());
@@ -269,17 +289,7 @@ QList<PlasmaWindowInterface*> PlasmaWindowManagementInterface::windows() const
     return d->windows;
 }
 
-void PlasmaWindowManagementInterface::unmapWindow(PlasmaWindowInterface *window)
-{
-    if (!window) {
-        return;
-    }
-    d->windows.removeOne(window);
-    Q_ASSERT(!d->windows.contains(window));
-    window->d->unmap();
-}
-
-void PlasmaWindowManagementInterface::setStackingOrder(const QVector<quint32>& stackingOrder)
+void PlasmaWindowManagementInterface::setStackingOrder(const QVector<quint32> &stackingOrder)
 {
     if (d->stackingOrder == stackingOrder) {
         return;
@@ -288,6 +298,15 @@ void PlasmaWindowManagementInterface::setStackingOrder(const QVector<quint32>& s
     d->sendStackingOrderChanged();
 }
 
+void PlasmaWindowManagementInterface::setStackingOrderUuids(const QVector<QString> &stackingOrderUuids)
+{
+    if (d->stackingOrderUuids == stackingOrderUuids) {
+        return;
+    }
+    d->stackingOrderUuids = stackingOrderUuids;
+    d->sendStackingOrderUuidsChanged();
+}
+
 void PlasmaWindowManagementInterface::setPlasmaVirtualDesktopManagementInterface(PlasmaVirtualDesktopManagementInterface *manager)
 {
     if (d->plasmaVirtualDesktopManagementInterface == manager) {
@@ -311,14 +330,7 @@ PlasmaWindowInterfacePrivate::PlasmaWindowInterfacePrivate(PlasmaWindowManagemen
 
 PlasmaWindowInterfacePrivate::~PlasmaWindowInterfacePrivate()
 {
-    destroyed = true;
-    const auto clientResources = resourceMap();
-    for (auto resource : clientResources) {
-        if(!unmapped) {
-            send_unmapped(resource->handle);
-        }
-        wl_resource_destroy(resource->handle);
-    }
+    unmap();
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_destroy(Resource *resource)
@@ -326,15 +338,6 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_destroy(Resource *resou
     wl_resource_destroy(resource->handle);
 }
 
-void PlasmaWindowInterfacePrivate::org_kde_plasma_window_destroy_resource(Resource *resource)
-{
-    Q_UNUSED(resource)
-    // Auto destruct when all resources gone
-    if (unmapped && resourceMap().isEmpty() && !destroyed) { 
-        delete q;
-    }
-}
-
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_bind_resource(Resource *resource)
 {
 
@@ -343,6 +346,11 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_bind_resource(Resource
     for (const auto &desk : plasmaVirtualDesktops) {
         send_virtual_desktop_entered(resource->handle, desk);
     }
+    for (const auto &activity : plasmaActivities) {
+        if (resource->version() >= ORG_KDE_PLASMA_WINDOW_ACTIVITY_ENTERED_SINCE_VERSION) {
+            send_activity_entered(resource->handle, activity);
+        }
+    }
     if (!m_appId.isEmpty()) {
         send_app_id_changed(resource->handle, m_appId);
     }
@@ -358,7 +366,7 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_bind_resource(Resource
     send_state_changed(resource->handle, m_state);
     if (!m_themedIconName.isEmpty()) {
         send_themed_icon_name_changed(resource->handle, m_themedIconName);
-    } else {
+    } else if (!m_icon.isNull()) {
         if (resource->version() >= ORG_KDE_PLASMA_WINDOW_ICON_CHANGED_SINCE_VERSION) {
             send_icon_changed(resource->handle);
         }
@@ -366,10 +374,6 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_bind_resource(Resource
 
     send_parent_window(resource->handle, resourceForParent(parentWindow, resource));
 
-    if (unmapped) {
-        send_unmapped(resource->handle);
-    }
-
     if (geometry.isValid() && resource->version() >= ORG_KDE_PLASMA_WINDOW_GEOMETRY_SINCE_VERSION) {
         send_geometry(resource->handle, geometry.x(), geometry.y(), geometry.width(), geometry.height());
     }
@@ -450,19 +454,31 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_get_icon(Resource *reso
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_enter_virtual_desktop(Resource *resource, const QString &id)
 {
     Q_UNUSED(resource)
-    emit q->enterPlasmaVirtualDesktopRequested(id);
+    Q_EMIT q->enterPlasmaVirtualDesktopRequested(id);
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_enter_new_virtual_desktop(Resource *resource)
 {
     Q_UNUSED(resource)
-    emit q->enterNewPlasmaVirtualDesktopRequested();
+    Q_EMIT q->enterNewPlasmaVirtualDesktopRequested();
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_leave_virtual_desktop(Resource *resource, const QString &id)
 {
     Q_UNUSED(resource)
-    emit q->leavePlasmaVirtualDesktopRequested(id);
+    Q_EMIT q->leavePlasmaVirtualDesktopRequested(id);
+}
+
+void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_enter_activity(Resource *resource, const QString &id)
+{
+    Q_UNUSED(resource)
+    Q_EMIT q->enterPlasmaActivityRequested(id);
+}
+
+void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_leave_activity(Resource *resource, const QString &id)
+{
+    Q_UNUSED(resource)
+    Q_EMIT q->leavePlasmaActivityRequested(id);
 }
 
 void PlasmaWindowInterfacePrivate::setTitle(const QString &title)
@@ -493,16 +509,15 @@ void PlasmaWindowInterfacePrivate::setVirtualDesktop(quint32 desktop)
 
 void PlasmaWindowInterfacePrivate::unmap()
 {
+    if (unmapped) {
+        return;
+    }
     unmapped = true;
     const auto clientResources = resourceMap();
 
     for (auto resource : clientResources) {
         send_unmapped(resource->handle);
     }
-
-    if (clientResources.isEmpty()) {
-        delete q;
-    }
 }
 
 void PlasmaWindowInterfacePrivate::setState(org_kde_plasma_window_management_state flag, bool set)
@@ -604,83 +619,83 @@ void PlasmaWindowInterfacePrivate::setApplicationMenuPaths(const QString &servic
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_close(Resource *resource)
 {
     Q_UNUSED(resource)
-    emit q->closeRequested();
+    Q_EMIT q->closeRequested();
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_move(Resource *resource)
 {
     Q_UNUSED(resource)
-    emit q->moveRequested();
+    Q_EMIT q->moveRequested();
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_request_resize(Resource *resource)
 {
     Q_UNUSED(resource)
-    emit q->resizeRequested();
+    Q_EMIT q->resizeRequested();
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_set_virtual_desktop(Resource *resource, uint32_t number)
 {
     Q_UNUSED(resource)
-    emit q->virtualDesktopRequested(number);
+    Q_EMIT q->virtualDesktopRequested(number);
 }
 
 void PlasmaWindowInterfacePrivate::org_kde_plasma_window_set_state(Resource *resource, uint32_t flags, uint32_t state)
 {
     Q_UNUSED(resource)
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ACTIVE) {
-        emit q->activeRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ACTIVE);
+        Q_EMIT q->activeRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_ACTIVE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZED) {
-        emit q->minimizedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZED);
+        Q_EMIT q->minimizedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZED);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZED) {
-        emit q->maximizedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZED);
+        Q_EMIT q->maximizedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZED);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREEN) {
-        emit q->fullscreenRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREEN);
+        Q_EMIT q->fullscreenRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREEN);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE) {
-        emit q->keepAboveRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE);
+        Q_EMIT q->keepAboveRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_ABOVE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW) {
-        emit q->keepBelowRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW);
+        Q_EMIT q->keepBelowRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_KEEP_BELOW);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_DEMANDS_ATTENTION) {
-        emit q->demandsAttentionRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_DEMANDS_ATTENTION);
+        Q_EMIT q->demandsAttentionRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_DEMANDS_ATTENTION);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_CLOSEABLE) {
-        emit q->closeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_CLOSEABLE);
+        Q_EMIT q->closeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_CLOSEABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZABLE) {
-        emit q->minimizeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZABLE);
+        Q_EMIT q->minimizeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MINIMIZABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZABLE) {
-        emit q->maximizeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZABLE);
+        Q_EMIT q->maximizeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MAXIMIZABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE) {
-        emit q->fullscreenableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE);
+        Q_EMIT q->fullscreenableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_FULLSCREENABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR) {
-        emit q->skipTaskbarRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR);
+        Q_EMIT q->skipTaskbarRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPTASKBAR);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPSWITCHER) {
-        emit q->skipSwitcherRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPSWITCHER);
+        Q_EMIT q->skipSwitcherRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SKIPSWITCHER);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADEABLE) {
-        emit q->shadeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADEABLE);
+        Q_EMIT q->shadeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADEABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADED) {
-        emit q->shadedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADED);
+        Q_EMIT q->shadedRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADED);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MOVABLE) {
-        emit q->movableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MOVABLE);
+        Q_EMIT q->movableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_MOVABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_RESIZABLE) {
-        emit q->resizableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_RESIZABLE);
+        Q_EMIT q->resizableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_RESIZABLE);
     }
     if (flags & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_VIRTUAL_DESKTOP_CHANGEABLE) {
-        emit q->virtualDesktopChangeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_VIRTUAL_DESKTOP_CHANGEABLE);
+        Q_EMIT q->virtualDesktopChangeableRequested(state & ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_VIRTUAL_DESKTOP_CHANGEABLE);
     }
 }
 
@@ -698,10 +713,10 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_set_minimized_geometry(
     }
 
     minimizedGeometries[panelSurface] = QRect(x, y, width, height);
-    emit q->minimizedGeometriesChanged();
+    Q_EMIT q->minimizedGeometriesChanged();
     QObject::connect(panelSurface, &QObject::destroyed, q, [this, panelSurface] () {
         if (minimizedGeometries.remove(panelSurface)) {
-            emit q->minimizedGeometriesChanged();
+            Q_EMIT q->minimizedGeometriesChanged();
         }
     });
 }
@@ -718,7 +733,7 @@ void PlasmaWindowInterfacePrivate::org_kde_plasma_window_unset_minimized_geometr
         return;
     }
     minimizedGeometries.remove(panelSurface);
-    emit q->minimizedGeometriesChanged();
+    Q_EMIT q->minimizedGeometriesChanged();
 }
 
 PlasmaWindowInterface::PlasmaWindowInterface(PlasmaWindowManagementInterface *wm, QObject *parent)
@@ -751,7 +766,7 @@ void PlasmaWindowInterface::setVirtualDesktop(quint32 desktop)
 
 void PlasmaWindowInterface::unmap()
 {
-    d->wm->unmapWindow(this);
+    d->unmap();
 }
 
 QHash<SurfaceInterface*, QRect>  PlasmaWindowInterface::minimizedGeometries() const
@@ -920,6 +935,41 @@ QStringList PlasmaWindowInterface::plasmaVirtualDesktops() const
     return d->plasmaVirtualDesktops;
 }
 
+void PlasmaWindowInterface::addPlasmaActivity(const QString &id)
+{
+    if (d->plasmaActivities.contains(id)) {
+        return;
+    }
+
+    d->plasmaActivities << id;
+
+    const auto clientResources = d->resourceMap();
+    for (auto resource : clientResources) {
+        if (resource->version() >= ORG_KDE_PLASMA_WINDOW_ACTIVITY_ENTERED_SINCE_VERSION) {
+            d->send_activity_entered(resource->handle, id);
+        }
+    }
+}
+
+void PlasmaWindowInterface::removePlasmaActivity(const QString &id)
+{
+    if (!d->plasmaActivities.removeOne(id)) {
+        return;
+    }
+
+    const auto clientResources = d->resourceMap();
+    for (auto resource : clientResources) {
+        if (resource->version() >= ORG_KDE_PLASMA_WINDOW_ACTIVITY_LEFT_SINCE_VERSION) {
+            d->send_activity_left(resource->handle, id);
+        }
+    }
+}
+
+QStringList PlasmaWindowInterface::plasmaActivities() const
+{
+    return d->plasmaActivities;
+}
+
 void PlasmaWindowInterface::setShadeable(bool set)
 {
     d->setState(ORG_KDE_PLASMA_WINDOW_MANAGEMENT_STATE_SHADEABLE, set);
@@ -965,4 +1015,9 @@ quint32 PlasmaWindowInterface::internalId() const
     return d->windowId;
 }
 
+QString PlasmaWindowInterface::uuid() const
+{
+    return d->uuid;
+}
+
 }
