@@ -3,8 +3,7 @@
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
-#ifndef WAYLAND_SERVER_PRIMARY_SELECTION_DEVICE_MANAGER_INTERFACE_H
-#define WAYLAND_SERVER_PRIMARY_SELECTION_DEVICE_MANAGER_INTERFACE_H
+#pragma once
 
 #include <QObject>
 
@@ -21,7 +20,7 @@ class PrimarySelectionDeviceV1Interface;
 /**
  * @brief Represents the Global for zwp_primary_selection_manager_v1 interface.
  *
- **/
+ */
 class KWAYLANDSERVER_EXPORT PrimarySelectionDeviceManagerV1Interface : public QObject
 {
     Q_OBJECT
@@ -39,5 +38,3 @@ private:
 };
 
 }
-
-#endif
