@@ -3,8 +3,7 @@
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
-#ifndef WAYLAND_SERVER_PRIMARY_SELECTION_SOURCE_INTERFACE_H
-#define WAYLAND_SERVER_PRIMARY_SELECTION_SOURCE_INTERFACE_H
+#pragma once
 
 #include "abstract_data_source.h"
 
@@ -20,7 +19,7 @@ class PrimarySelectionSourceV1InterfacePrivate;
 /**
  * @brief Represents the Resource for the zwp_primary_selection_source_v1 interface.
  * Lifespan is mapped to the underlying object
- **/
+ */
 class KWAYLANDSERVER_EXPORT PrimarySelectionSourceV1Interface : public AbstractDataSource
 {
     Q_OBJECT
@@ -45,5 +44,3 @@ private:
 }
 
 Q_DECLARE_METATYPE(KWaylandServer::PrimarySelectionSourceV1Interface*)
-
-#endif
