@@ -12,7 +12,7 @@
 #include "display.h"
 #include "idleinhibit_v1_interface_p.h"
 #include "pointerconstraints_v1_interface_p.h"
-#include "region_interface.h"
+#include "region_interface_p.h"
 #include "subcompositor_interface.h"
 #include "subsurface_interface_p.h"
 #include "surfacerole_p.h"
@@ -83,15 +83,9 @@ void SurfaceInterfacePrivate::addChild(SubSurfaceInterface *child)
     pending.children.append(child);
     cached.children.append(child);
     current.children.append(child);
-
     child->surface()->setOutputs(outputs);
-
-    emit q->childSubSurfaceAdded(child);
-    emit q->subSurfaceTreeChanged();
-    QObject::connect(child, &SubSurfaceInterface::positionChanged, q, &SurfaceInterface::subSurfaceTreeChanged);
-    QObject::connect(child->surface(), &SurfaceInterface::damaged, q, &SurfaceInterface::subSurfaceTreeChanged);
-    QObject::connect(child->surface(), &SurfaceInterface::unmapped, q, &SurfaceInterface::subSurfaceTreeChanged);
-    QObject::connect(child->surface(), &SurfaceInterface::subSurfaceTreeChanged, q, &SurfaceInterface::subSurfaceTreeChanged);
+    Q_EMIT q->childSubSurfaceAdded(child);
+    Q_EMIT q->childSubSurfacesChanged();
 }
 
 void SurfaceInterfacePrivate::removeChild(SubSurfaceInterface *child)
@@ -100,14 +94,8 @@ void SurfaceInterfacePrivate::removeChild(SubSurfaceInterface *child)
     pending.children.removeAll(child);
     cached.children.removeAll(child);
     current.children.removeAll(child);
-    emit q->childSubSurfaceRemoved(child);
-    emit q->subSurfaceTreeChanged();
-    QObject::disconnect(child, &SubSurfaceInterface::positionChanged, q, &SurfaceInterface::subSurfaceTreeChanged);
-    if (child->surface()) {
-        QObject::disconnect(child->surface(), &SurfaceInterface::damaged, q, &SurfaceInterface::subSurfaceTreeChanged);
-        QObject::disconnect(child->surface(), &SurfaceInterface::unmapped, q, &SurfaceInterface::subSurfaceTreeChanged);
-        QObject::disconnect(child->surface(), &SurfaceInterface::subSurfaceTreeChanged, q, &SurfaceInterface::subSurfaceTreeChanged);
-    }
+    Q_EMIT q->childSubSurfaceRemoved(child);
+    Q_EMIT q->childSubSurfacesChanged();
 }
 
 bool SurfaceInterfacePrivate::raiseChild(SubSurfaceInterface *subsurface, SurfaceInterface *sibling)
@@ -122,8 +110,8 @@ bool SurfaceInterfacePrivate::raiseChild(SubSurfaceInterface *subsurface, Surfac
     }
     if (sibling == q) {
         // it's to the parent, so needs to become last item
-        pending.children.append(*it);
         pending.children.erase(it);
+        pending.children.append(subsurface);
         pending.childrenChanged = true;
         return true;
     }
@@ -218,7 +206,7 @@ void SurfaceInterfacePrivate::installPointerConstraint(LockedPointerV1Interface
         constrainsOneShotConnection = QMetaObject::Connection();
         QObject::disconnect(constrainsUnboundConnection);
         constrainsUnboundConnection = QMetaObject::Connection();
-        emit q->pointerConstraintsChanged();
+        Q_EMIT q->pointerConstraintsChanged();
     };
 
     if (lock->lifeTime() == LockedPointerV1Interface::LifeTime::OneShot) {
@@ -232,7 +220,7 @@ void SurfaceInterfacePrivate::installPointerConstraint(LockedPointerV1Interface
         );
     }
     constrainsUnboundConnection = QObject::connect(lock, &LockedPointerV1Interface::destroyed, q, cleanUp);
-    emit q->pointerConstraintsChanged();
+    Q_EMIT q->pointerConstraintsChanged();
 }
 
 void SurfaceInterfacePrivate::installPointerConstraint(ConfinedPointerV1Interface *confinement)
@@ -248,7 +236,7 @@ void SurfaceInterfacePrivate::installPointerConstraint(ConfinedPointerV1Interfac
         constrainsOneShotConnection = QMetaObject::Connection();
         QObject::disconnect(constrainsUnboundConnection);
         constrainsUnboundConnection = QMetaObject::Connection();
-        emit q->pointerConstraintsChanged();
+        Q_EMIT q->pointerConstraintsChanged();
     };
 
     if (confinement->lifeTime() == ConfinedPointerV1Interface::LifeTime::OneShot) {
@@ -262,7 +250,7 @@ void SurfaceInterfacePrivate::installPointerConstraint(ConfinedPointerV1Interfac
         );
     }
     constrainsUnboundConnection = QObject::connect(confinement, &ConfinedPointerV1Interface::destroyed, q, cleanUp);
-    emit q->pointerConstraintsChanged();
+    Q_EMIT q->pointerConstraintsChanged();
 }
 
 void SurfaceInterfacePrivate::installIdleInhibitor(IdleInhibitorV1Interface *inhibitor)
@@ -272,18 +260,18 @@ void SurfaceInterfacePrivate::installIdleInhibitor(IdleInhibitorV1Interface *inh
         [this, inhibitor] {
             idleInhibitors.removeOne(inhibitor);
             if (idleInhibitors.isEmpty()) {
-                emit q->inhibitsIdleChanged();
+                Q_EMIT q->inhibitsIdleChanged();
             }
         }
     );
     if (idleInhibitors.count() == 1) {
-        emit q->inhibitsIdleChanged();
+        Q_EMIT q->inhibitsIdleChanged();
     }
 }
 
 void SurfaceInterfacePrivate::surface_destroy_resource(Resource *)
 {
-    emit q->aboutToBeDestroyed();
+    Q_EMIT q->aboutToBeDestroyed();
     delete q;
 }
 
@@ -622,24 +610,24 @@ void SurfaceInterfacePrivate::swapStates(State *source, State *target, bool emit
     bufferToSurfaceMatrix = surfaceToBufferMatrix.inverted();
     inputRegion = target->input & QRect(QPoint(0, 0), target->size);
     if (opaqueRegionChanged) {
-        emit q->opaqueChanged(target->opaque);
+        Q_EMIT q->opaqueChanged(target->opaque);
     }
     if (oldInputRegion != inputRegion) {
-        emit q->inputChanged(inputRegion);
+        Q_EMIT q->inputChanged(inputRegion);
     }
     if (scaleFactorChanged) {
-        emit q->bufferScaleChanged(target->bufferScale);
+        Q_EMIT q->bufferScaleChanged(target->bufferScale);
     }
     if (transformChanged) {
-        emit q->bufferTransformChanged(target->bufferTransform);
+        Q_EMIT q->bufferTransformChanged(target->bufferTransform);
     }
     if (visibilityChanged) {
         if (target->buffer) {
             subSurfaceIsMapped = true;
-            emit q->mapped();
+            Q_EMIT q->mapped();
         } else {
             subSurfaceIsMapped = false;
-            emit q->unmapped();
+            Q_EMIT q->unmapped();
         }
     }
     if (bufferChanged) {
@@ -647,41 +635,32 @@ void SurfaceInterfacePrivate::swapStates(State *source, State *target, bool emit
             const QRegion windowRegion = QRegion(0, 0, q->size().width(), q->size().height());
             const QRegion bufferDamage = q->mapFromBuffer(target->bufferDamage);
             target->damage = windowRegion.intersected(target->damage.united(bufferDamage));
-            trackedDamage |= target->damage;
-            emit q->damaged(target->damage);
-            // workaround for https://bugreports.qt.io/browse/QTBUG-52092
-            // if the surface is a sub-surface, but the main surface is not yet mapped, fake frame rendered
-            if (subSurface) {
-                const auto mainSurface = subSurface->mainSurface();
-                if (!mainSurface || !mainSurface->buffer()) {
-                    q->frameRendered(0);
-                }
-            }
+            Q_EMIT q->damaged(target->damage);
         }
     }
     if (surfaceToBufferMatrix != oldSurfaceToBufferMatrix) {
-        emit q->surfaceToBufferMatrixChanged();
+        Q_EMIT q->surfaceToBufferMatrixChanged();
     }
     if (bufferSize != oldBufferSize) {
-        emit q->bufferSizeChanged();
+        Q_EMIT q->bufferSizeChanged();
     }
     if (target->size != oldSize) {
-        emit q->sizeChanged();
+        Q_EMIT q->sizeChanged();
     }
     if (shadowChanged) {
-        emit q->shadowChanged();
+        Q_EMIT q->shadowChanged();
     }
     if (blurChanged) {
-        emit q->blurChanged();
+        Q_EMIT q->blurChanged();
     }
     if (contrastChanged) {
-        emit q->contrastChanged();
+        Q_EMIT q->contrastChanged();
     }
     if (slideChanged) {
-        emit q->slideOnShowHideChanged();
+        Q_EMIT q->slideOnShowHideChanged();
     }
     if (childrenChanged) {
-        emit q->subSurfaceTreeChanged();
+        Q_EMIT q->childSubSurfacesChanged();
     }
 }
 
@@ -702,7 +681,7 @@ void SurfaceInterfacePrivate::commit()
         role->commit();
     }
 
-    emit q->committed();
+    Q_EMIT q->committed();
 }
 
 QRegion SurfaceInterface::damage() const
@@ -817,16 +796,6 @@ bool SurfaceInterface::isMapped() const
     return d->current.buffer != nullptr;
 }
 
-QRegion SurfaceInterface::trackedDamage() const
-{
-    return d->trackedDamage;
-}
-
-void SurfaceInterface::resetTrackedDamage()
-{
-    d->trackedDamage = QRegion();
-}
-
 QVector<OutputInterface *> SurfaceInterface::outputs() const
 {
     return d->outputs;
@@ -858,7 +827,7 @@ void SurfaceInterface::setOutputs(const QVector<OutputInterface *> &outputs)
         for (wl_resource *outputResource : resources) {
             d->send_enter(outputResource);
         }
-        d->outputDestroyedConnections[o] = connect(o, &Global::aboutToDestroyGlobal, this, [this, o] {
+        d->outputDestroyedConnections[o] = connect(o, &OutputInterface::removed, this, [this, o] {
             auto outputs = d->outputs;
             if (outputs.removeOne(o)) {
                 setOutputs(outputs);
@@ -999,4 +968,32 @@ void SurfaceInterface::handleBufferRemoved(BufferInterface *buffer)
     }
 }
 
+QPointF SurfaceInterface::mapToChild(SurfaceInterface *child, const QPointF &point) const
+{
+    QPointF local = point;
+    SurfaceInterface *surface = child;
+
+    while (true) {
+        if (surface == this) {
+            return local;
+        }
+
+        SubSurfaceInterface *subsurface = surface->subSurface();
+        if (Q_UNLIKELY(!subsurface)) {
+            return QPointF();
+        }
+
+        local -= subsurface->position();
+        surface = subsurface->parentSurface();
+    }
+
+    return QPointF();
+}
+
+
+QSize SurfaceInterface::bufferSize() const
+{
+    return d->bufferSize;
+}
+
 } // namespace KWaylandServer
