@@ -11,8 +11,9 @@
 namespace KWaylandServer
 {
 
-SurfaceRole::SurfaceRole(SurfaceInterface *surface)
+SurfaceRole::SurfaceRole(SurfaceInterface *surface, const QByteArray &name)
     : m_surface(surface)
+    , m_name(name)
 {
     SurfaceInterfacePrivate *surfacePrivate = SurfaceInterfacePrivate::get(surface);
     surfacePrivate->role = this;
@@ -27,6 +28,11 @@ SurfaceRole::~SurfaceRole()
     }
 }
 
+QByteArray SurfaceRole::name() const
+{
+    return m_name;
+}
+
 SurfaceRole *SurfaceRole::get(SurfaceInterface *surface)
 {
     if (surface) {
