@@ -3,9 +3,7 @@
 
     SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
-
-#ifndef KWAYLAND_SERVER_SURFACEROLE_P_H
-#define KWAYLAND_SERVER_SURFACEROLE_P_H
+#pragma once
 
 #include <QByteArray>
 #include <QPointer>
@@ -39,5 +37,3 @@ private:
 };
 
 }
-
-#endif // KWAYLAND_SERVER_SURFACEROLE_P_H
